
import { TypeHierarchyModel, TypeHierarchyDirection } from '@codingame/monaco-vscode-7f39b6f1-3542-5430-8760-0f404d7a7cee-common/vscode/vs/workbench/contrib/typeHierarchy/common/typeHierarchy';
import { CancellationToken } from 'vscode/vscode/vs/base/common/cancellation';
import { createMatches } from 'vscode/vscode/vs/base/common/filters';
import { IconLabel } from 'vscode/vscode/vs/base/browser/ui/iconLabel/iconLabel';
import { SymbolTag, SymbolKinds } from 'vscode/vscode/vs/editor/common/languages';
import { compare } from 'vscode/vscode/vs/base/common/strings';
import { Range } from 'vscode/vscode/vs/editor/common/core/range';
import { localize } from 'vscode/vscode/vs/nls';
import { ThemeIcon } from 'vscode/vscode/vs/base/common/themables';

class Type {
    constructor(item, model, parent) {
        this.item = item;
        this.model = model;
        this.parent = parent;
    }
    static compare(a, b) {
        let res = compare(( a.item.uri.toString()), ( b.item.uri.toString()));
        if (res === 0) {
            res = Range.compareRangesUsingStarts(a.item.range, b.item.range);
        }
        return res;
    }
}
class DataSource {
    constructor(getDirection) {
        this.getDirection = getDirection;
    }
    hasChildren() {
        return true;
    }
    async getChildren(element) {
        if (element instanceof TypeHierarchyModel) {
            return ( element.roots.map(root => ( new Type(root, element, undefined))));
        }
        const { model, item } = element;
        if (this.getDirection() === TypeHierarchyDirection.Supertypes) {
            return ( (await model.provideSupertypes(item, CancellationToken.None)).map(item => {
                return ( new Type(item, model, element));
            }));
        }
        else {
            return ( (await model.provideSubtypes(item, CancellationToken.None)).map(item => {
                return ( new Type(item, model, element));
            }));
        }
    }
}
class Sorter {
    compare(element, otherElement) {
        return Type.compare(element, otherElement);
    }
}
class IdentityProvider {
    constructor(getDirection) {
        this.getDirection = getDirection;
    }
    getId(element) {
        let res = this.getDirection() + JSON.stringify(element.item.uri) + JSON.stringify(element.item.range);
        if (element.parent) {
            res += this.getId(element.parent);
        }
        return res;
    }
}
class TypeRenderingTemplate {
    constructor(icon, label) {
        this.icon = icon;
        this.label = label;
    }
}
class TypeRenderer {
    constructor() {
        this.templateId = TypeRenderer.id;
    }
    static { this.id = 'TypeRenderer'; }
    renderTemplate(container) {
        container.classList.add('typehierarchy-element');
        const icon = document.createElement('div');
        container.appendChild(icon);
        const label = ( new IconLabel(container, { supportHighlights: true }));
        return ( new TypeRenderingTemplate(icon, label));
    }
    renderElement(node, _index, template) {
        const { element, filterData } = node;
        const deprecated = element.item.tags?.includes(SymbolTag.Deprecated);
        template.icon.classList.add('inline', ...ThemeIcon.asClassNameArray(SymbolKinds.toIcon(element.item.kind)));
        template.label.setLabel(element.item.name, element.item.detail, { labelEscapeNewLines: true, matches: createMatches(filterData), strikethrough: deprecated });
    }
    disposeTemplate(template) {
        template.label.dispose();
    }
}
class VirtualDelegate {
    getHeight(_element) {
        return 22;
    }
    getTemplateId(_element) {
        return TypeRenderer.id;
    }
}
class AccessibilityProvider {
    constructor(getDirection) {
        this.getDirection = getDirection;
    }
    getWidgetAriaLabel() {
        return localize(10409, "Type Hierarchy");
    }
    getAriaLabel(element) {
        if (this.getDirection() === TypeHierarchyDirection.Supertypes) {
            return localize(10410, "supertypes of {0}", element.item.name);
        }
        else {
            return localize(10411, "subtypes of {0}", element.item.name);
        }
    }
}

export { AccessibilityProvider, DataSource, IdentityProvider, Sorter, Type, TypeRenderer, VirtualDelegate };
